"use strict";
/*
 * @Author: Lisc
 * @Date: 2022-04-03 16:22:01
 * @Description: Axios类是Axios接口的底层实现,但是并不是严格意义的实现类
 * 因为request接口中只有config一个参数,而实现类有两个
 * 这样做是为了达到外界调用request方法时只能传入config对象的效果
 */
Object.defineProperty(exports, "__esModule", { value: true });
var dispatchRequest_1 = require("./dispatchRequest");
var interceptorManager_1 = require("./interceptorManager");
var mergeConfig_1 = require("./mergeConfig");
var Axios = /** @class */ (function () {
    function Axios(initConfig) {
        this.defaults = initConfig;
        this.interceptors = {
            request: new interceptorManager_1.default(),
            response: new interceptorManager_1.default(),
        };
    }
    /**
     * @description: 直接以函数形式(两种方式)调用axios 和 调用axios的别名(request,get,post等)时,底层的实现方法
     * @param {any} urlOrConfig
     * @param {AxiosRequestConfig} config
     * @return {*}
     */
    Axios.prototype.request = function (urlOrConfig, config) {
        // 支持直接传入配置对象和传入url和配置对象两种调用方式
        if (typeof urlOrConfig === 'string') {
            if (!config) {
                config = {};
            }
            config.url = urlOrConfig;
        }
        else {
            config = urlOrConfig;
        }
        config = (0, mergeConfig_1.default)(this.defaults, config);
        config.method = config.method.toLowerCase();
        //拦截器链式调用
        var chain = [
            // 初始化Promise链,最开始只有发送请求的resolve
            {
                resolve: dispatchRequest_1.default,
            },
        ];
        // 遍历拦截器并添加到Promise链
        this.interceptors.request.forEach(function (interceptor) {
            // 请求拦截器后添加的先执行
            chain.unshift(interceptor);
        });
        this.interceptors.response.forEach(function (interceptor) {
            // 响应拦截器先添加的先执行
            chain.push(interceptor);
        });
        //初始化一个resolve了config对象的promise对象
        //这里resolve了config对象,第一个then函数的resolve回调才能拿到config
        //并对config进行加工
        var promise = Promise.resolve(config);
        //链式调用
        while (chain.length) {
            var _a = chain.shift(), resolve = _a.resolve, reject = _a.reject;
            promise = promise.then(resolve, reject);
        }
        // 最终返回给用户的promise
        return promise;
    };
    Axios.prototype.get = function (url, config) {
        return this._requestWithoutData('get', url, config);
    };
    Axios.prototype.delete = function (url, config) {
        return this._requestWithoutData('delete', url, config);
    };
    Axios.prototype.head = function (url, config) {
        return this._requestWithoutData('head', url, config);
    };
    Axios.prototype.options = function (url, config) {
        return this._requestWithoutData('options', url, config);
    };
    Axios.prototype.post = function (url, data, config) {
        return this._requestWithData('post', url, data, config);
    };
    Axios.prototype.put = function (url, data, config) {
        return this._requestWithData('put', url, data, config);
    };
    Axios.prototype.patch = function (url, data, config) {
        return this._requestWithData('patch', url, data, config);
    };
    Axios.prototype.getUri = function (config) {
        config = (0, mergeConfig_1.default)(this.defaults, config);
        return (0, dispatchRequest_1.transformUrl)(config);
    };
    /**
     * @description: 支持get、delete、head和options方法
     * @param {Method} method
     * @param {string} url
     * @param {AxiosRequestConfig} config
     * @return {*}
     */
    Axios.prototype._requestWithoutData = function (method, url, config) {
        return this.request(Object.assign(config || {}, {
            method: method,
            url: url,
        }));
    };
    /**
     * @description: 支持post、put、patch方法
     * @param {*}
     * @return {*}
     */
    Axios.prototype._requestWithData = function (method, url, data, config) {
        return this.request(Object.assign(config || {}, {
            method: method,
            url: url,
            data: data,
        }));
    };
    return Axios;
}());
exports.default = Axios;
//# sourceMappingURL=Axios.js.map