"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformUrl = void 0;
var url_1 = require("../utils/url");
var headers_1 = require("../utils/headers");
var xhr_1 = require("./xhr");
var transform_1 = require("./transform");
/**
 * @description:
 * @param {AxiosRequestConfig} config
 * @return {*}
 */
function dispatchRequest(config) {
    // cancelToken已经被使用过，再去携带相同的cancelToken发送请求没有意义
    throwIfCanceled(config);
    processRequestConfig(config);
    // 返回带有xhr请求结果的Promise对象
    return (0, xhr_1.default)(config).then(function (resp) {
        return transformResponseData(resp);
    }, function (e) {
        if (e && e.response) {
            e.response = transformResponseData(e.response);
        }
        return Promise.reject(e);
    });
}
exports.default = dispatchRequest;
function processRequestConfig(config) {
    config.url = transformUrl(config);
    config.data = transformRequestData(config);
    config.headers = (0, headers_1.flattenHeaders)(config.headers, config.method);
}
/**
 * @description: 根据用户配置的baseURL，url，params和paramsSerializer得到完整的请求url
 * @param {AxiosRequestConfig} config
 * @return {*}
 */
function transformUrl(config) {
    var url = config.url, params = config.params, paramsSerializer = config.paramsSerializer, baseURL = config.baseURL;
    if (baseURL && !(0, url_1.isAbsoluteURL)(url)) {
        url = (0, url_1.combineURL)(baseURL, url);
    }
    return (0, url_1.getUrlWithParams)(url, params, paramsSerializer);
}
exports.transformUrl = transformUrl;
/**
 * @description: 根据用户设置的请求数据(请求体data)处理函数,对请求数据进行处理
 * 默认的请求数据处理函数为:如果请求数据为普通对象,将其转换成Json字符串,并设置其Content-Type为application/json
 * @param {AxiosRequestConfig} config
 * @return {*}
 */
function transformRequestData(config) {
    return (0, transform_1.default)(config.data, config.headers, config.transformRequest);
}
/**
 * @description: 根据用户设置的响应数据处理函数,对响应数据进行处理
 * 默认的响应数据处理函数为:如果响应数据为Json字符串,将其转换成Json对象
 * @param {AxiosRequestConfig} config
 * @return {*}
 */
function transformResponseData(resp) {
    resp.data = (0, transform_1.default)(resp.data, resp.headers, resp.config.transformResponse);
    return resp;
}
function throwIfCanceled(config) {
    if (config.cancelToken) {
        config.cancelToken.throwIfRequested();
    }
}
//# sourceMappingURL=dispatchRequest.js.map