/*
 * @Author: Lisc
 * @Date: 2022-03-30 21:09:13
 * @Description: 工具类
 */
var Tostring = Object.prototype.toString;
/**
 * @description: 判断一个对象是不是普通对象（是不是通过 "{}" 或者 "new Object" 创建的）
 * @param {any} o
 * @return {*}
 */
function isPlainObject(o) {
    return Tostring.call(o) === '[object Object]';
}
/**
 * @description: 判断一个类型是否为日期类型
 * @param {any} d
 * @return {*}
 */
function isDate(d) {
    return Tostring.call(d) === '[object Date]';
}
function isFormData(val) {
    return typeof val !== 'undefined' && val instanceof FormData;
}
function isURLSearchParams(val) {
    return typeof val !== 'undefined' && val instanceof URLSearchParams;
}
/**
 * @description: 将from对象的所有成员复制到to对象
 * @param {T} to
 * @param {U} from
 * @return {*}
 */
function extend(to, from) {
    for (var key in from) {
        to[key] = from[key];
    }
    return to;
}
/**
 * @description: 深拷贝,支持将多个对象的属性拷贝到一个对象中
 * @param {array} objs
 * @return {*}
 */
function deepCopy() {
    var objs = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        objs[_i] = arguments[_i];
    }
    var result = Object.create(null);
    objs.forEach(function (obj) {
        if (obj) {
            Object.keys(obj).forEach(function (key) {
                var val = obj[key];
                if (isPlainObject(val)) {
                    if (isPlainObject(result[key])) {
                        result[key] = deepCopy(result[key], val);
                    }
                    else {
                        result[key] = deepCopy(val);
                    }
                }
                else {
                    result[key] = val;
                }
            });
        }
    });
    return result;
}

/*
 * @Author: Lisc
 * @Date: 2022-03-30 20:59:08
 * @Description: 处理请求url路径
 */
/**
 * @description: 用于将一个对象通过 URL 进行传输
 * encodeURIComponent方法将特殊字符比如空格进行编码，
 * 不会对 ASCII 字母和数字进行编码，也不会对这些 ASCII 标点符号进行编码
 * @param {string} preVal
 * @return {string}
 */
function encode(preVal) {
    return encodeURIComponent(preVal)
        .replace(/%40/g, '@')
        .replace(/%3A/gi, ':')
        .replace(/%24/g, '$')
        .replace(/%2C/gi, ',')
        .replace(/%20/g, '+')
        .replace(/%5B/gi, '[')
        .replace(/%5D/gi, ']');
}
/**
 * @description: 拼接得到get请求的url
 * @param {string} url
 * @param {any} params
 * @return {*}
 */
function getUrlWithParams(url, params, paramsSerializer) {
    if (!params) {
        return url;
    }
    var serializedParams;
    // 如果用户自定义了参数序列化方式，使用用户自定义的
    if (paramsSerializer) {
        serializedParams = paramsSerializer(params);
    }
    else if (isURLSearchParams(params)) {
        // 用户传入的params是URLSearchParams类型，直接转为字符串
        serializedParams = params.toString();
    }
    else {
        // 根据用户配置的params对象，生成序列化后的参数字符串
        var parafragments_1 = [];
        // params实际上是一个对象
        Object.keys(params).forEach(function (key) {
            var val = params[key];
            // 参数值传了一个空值
            if (!val) {
                return;
            }
            // 将每个参数值转成数组方便进行统一处理
            var valArr = [];
            if (Array.isArray(val)) {
                valArr = val;
                key += '[]';
            }
            else {
                valArr = [val];
            }
            valArr.forEach(function (val) {
                if (isDate(val)) {
                    val = val.toISOString();
                }
                else if (isPlainObject(val)) {
                    val = JSON.stringify(val);
                }
                // 只有参数值为数组时会push多次
                parafragments_1.push("".concat(encode(key), "=").concat(encode(val)));
            });
        });
        serializedParams = parafragments_1.join('&');
    }
    // 去除url的锚点
    var index = url.indexOf('#');
    if (index !== -1) {
        url = url.slice(0, index);
    }
    // 将url和序列化后的参数字符串拼接
    if (serializedParams.length) {
        url = url + (url.indexOf('?') === -1 ? '?' : '&') + serializedParams;
    }
    return url;
}
function isURLSameOrigin(url) {
    var parsedOrigin = resolveURL(url);
    return (
    // host为主机和端口，hostname只有主机
    parsedOrigin.protocol === window.location.protocol && parsedOrigin.host === window.location.host);
}
function isAbsoluteURL(url) {
    return /^([a-z][a-z\d\+\-\.]*:)?\/\//i.test(url);
}
function combineURL(baseURL, relativeURL) {
    // 自动删除baseURL后边和relativeURL前边的一个或多个/
    return relativeURL ? baseURL.replace(/\/+$/, '') + '/' + relativeURL.replace(/^\/+/, '') : baseURL;
}
var urlParsingNode = document.createElement('a');
function resolveURL(url) {
    urlParsingNode.setAttribute('href', url);
    var protocol = urlParsingNode.protocol, host = urlParsingNode.host;
    return {
        protocol: protocol,
        host: host,
    };
}

/*
 * @Author: Lisc
 * @Date: 2022-03-31 10:39:00
 * @Description: 处理请求头和响应头
 */
/**
 * @description: 将用户传入的请求头中的key转换成标准格式
 * 例如 content-type 会被转换成 Content-Type
 * @param {any} headers
 * @param {string} normalizedHeaderName
 * @return {*}
 */
function normalizeHeaderName(headers, normalizedHeaderName) {
    if (!headers) {
        return;
    }
    Object.keys(headers).forEach(function (headerName) {
        if (headerName.toUpperCase() === normalizedHeaderName.toUpperCase() &&
            headerName !== normalizedHeaderName) {
            headers[normalizedHeaderName] = headers[headerName];
            delete headers[headerName];
        }
    });
}
/**
 * @description: xhr 中的data数据不接受对象形式，需要转为Json字符串
 * 而Json字符串会被当做普通文本处理，xhr默认设置的请求头为Content-Type为text/plain，
 * 这里需要将其修改成application/json
 * @param {any} headers
 * @param {any} data
 * @return {*}
 */
function setHeaders(headers, data) {
    var contentTypeStr = 'Content-Type';
    var jsonContentType = 'application/json';
    normalizeHeaderName(headers, contentTypeStr);
    if (isPlainObject(data)) {
        if (headers && !headers[contentTypeStr]) {
            headers[contentTypeStr] = jsonContentType;
        }
    }
    return headers;
}
/**
 * @description: 将字符串类型的响应头转成对象形式
 * @param {string} headers
 * @return {*}
 */
function parseHeaders(headers) {
    var parsedHeaders = Object.create(null);
    if (!headers) {
        return parsedHeaders;
    }
    headers.split('\r\n').forEach(function (line) {
        var _a = line.split(':'), key = _a[0], valuesArr = _a.slice(1);
        if (!key) {
            return;
        }
        key = key.trim().toLocaleLowerCase();
        var value = valuesArr.join(':').trim();
        parsedHeaders[key] = value;
    });
    return parsedHeaders;
}
function flattenHeaders(headers, method) {
    if (!headers) {
        return headers;
    }
    // 顺序不可颠倒,后边的优先级高
    var methodMergedHeaders = deepCopy(headers.common, headers[method]);
    var keysToDelete = [
        'common',
        'get',
        'delete',
        'head',
        'options',
        'post',
        'put',
        'patch',
    ];
    keysToDelete.forEach(function (key) {
        delete headers[key];
    });
    return deepCopy(methodMergedHeaders, headers);
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    if (typeof b !== "function" && b !== null)
        throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var AxiosError = /** @class */ (function (_super) {
    __extends(AxiosError, _super);
    function AxiosError(message, config, code, request, response) {
        var _this = _super.call(this, message) || this;
        _this.isAxiosError = true;
        _this.config = config;
        _this.code = code;
        _this.request = request;
        _this.response = response;
        Object.setPrototypeOf(_this, AxiosError.prototype);
        return _this;
    }
    return AxiosError;
}(Error));
function createError(message, config, code, request, response) {
    var error = new AxiosError(message, config, code, request, response);
    return error;
}

/*
 * @Author: Lisc
 * @Date: 2022-04-12 12:44:52
 * @Description:
 */
var cookie = {
    /**
     * @description: 遍历cookie得到指定cookie
     * @param {string} cookieName
     * @return {*}
     */
    // getCookie(cookieName: string): string | null {
    //   const strCookie = document.cookie
    //   const cookieList = strCookie.split(';')
    //   for (let i = 0; i < cookieList.length; i++) {
    //     const arr = cookieList[i].split('=')
    //     if (cookieName === arr[0].trim()) {
    //       return arr[1]
    //     }
    //   }
    //   return null
    // }
    /**
     * @description: 正则表达式根据cookie的key匹配cookie的value
     * @param {string} key
     * @return {*}
     */
    getCookie: function (key) {
        var str = "(^| )".concat(key, "=([^;]*)(;|$)");
        var reg = new RegExp(str);
        var arr = document.cookie.match(reg);
        if (!arr) {
            return null;
        }
        return arr[2]; //第2个小括号对应cookie的value
    }
};

/*
 * @Author: Lisc
 * @Date: 2022-03-30 16:36:33
 * @Description: 发送xhr请求
 */
function xhr(config) {
    return new Promise(function (resolve, reject) {
        var url = config.url, responseType = config.responseType, timeout = config.timeout, cancelToken = config.cancelToken, withCredentials = config.withCredentials, xsrfCookieName = config.xsrfCookieName, xsrfHeaderName = config.xsrfHeaderName, onDownloadProgress = config.onDownloadProgress, onUploadProgress = config.onUploadProgress, auth = config.auth, validateStatus = config.validateStatus, _a = config.method, method = _a === void 0 ? 'get' : _a, _b = config.headers, headers = _b === void 0 ? {} : _b, _c = config.data, data = _c === void 0 ? null : _c;
        var xhr = new XMLHttpRequest();
        // xhr.open(method.toUpperCase(), url!, true)
        xhr.open(method, url, true);
        configXhr();
        addEvents();
        xhr.send(data);
        /**
         * @description: 处理xhr的headers
         * @param {*}
         * @return {*}
         */
        function processHeaders() {
            if (isFormData(data)) {
                // defaults中将含有data请求的Content-Type全部设置为了application/x-www-form-urlencoded
                // 如果请求数据类型为FormData，需要删除我们默认设置的默认值，让浏览器根据请求数据自动设置为form-data，
                // 即multipart/form-data; boundary=----WebKitFormBoundaryfMam9AjKqWXMBjBF
                delete headers['Content-Type'];
            }
            // XSRF 防御
            if ((withCredentials || isURLSameOrigin(url)) && xsrfCookieName) {
                var xsrfToken = cookie.getCookie(xsrfCookieName);
                if (xsrfToken && xsrfHeaderName) {
                    headers[xsrfHeaderName] = xsrfToken;
                }
            }
            if (auth) {
                headers['Authorization'] = "Basic ".concat(btoa("".concat(auth.username, ":").concat(auth.password)));
            }
            // 给xhr设置请求头
            Object.keys(headers).forEach(function (headerName) {
                if (headerName.toLowerCase() === 'content-type' && data === null) {
                    delete headers[headerName];
                    return;
                }
                xhr.setRequestHeader(headerName, headers[headerName]);
            });
        }
        /**
         * @description: 处理xhr的cancelToken配置
         * @param {*}
         * @return {*}
         */
        function processCancel() {
            if (cancelToken) {
                cancelToken.promise.then(function (reason) {
                    xhr.abort();
                    reject(reason);
                });
            }
        }
        /**
         * @description: 配置xhr，设置xhr的各个属性
         * @param {*}
         * @return {*}
         */
        function configXhr() {
            if (responseType) {
                xhr.responseType = responseType;
            }
            if (timeout) {
                xhr.timeout = timeout;
            }
            if (withCredentials) {
                xhr.withCredentials = withCredentials;
            }
            processHeaders();
            processCancel();
        }
        /**
         * @description: 为xhr设置监听事件
         * @param {*}
         * @return {*}
         */
        function addEvents() {
            if (onDownloadProgress) {
                xhr.onprogress = onDownloadProgress;
            }
            if (onUploadProgress) {
                xhr.upload.onprogress = onUploadProgress;
            }
            xhr.onreadystatechange = function () {
                if (xhr.status === 0) {
                    return;
                }
                if (xhr.readyState !== 4) {
                    return;
                }
                // 成功获取响应
                var responseHeaders = parseHeaders(xhr.getAllResponseHeaders());
                var responseData = responseType && responseType !== 'text' ? xhr.response : xhr.responseText;
                var response = {
                    data: responseData,
                    status: xhr.status,
                    statusText: xhr.statusText,
                    headers: responseHeaders,
                    config: config,
                    request: xhr,
                };
                handleResponseStatus(response);
            };
            // 网络异常错误处理
            xhr.addEventListener('error', function () {
                reject(createError('Network Error', config, null, xhr));
            });
            // 超时异常处理
            xhr.addEventListener('timeout', function () {
                reject(createError("Timeout of ".concat(timeout, " ms exceeded"), config, 'ECONNABORTED', xhr));
            });
        }
        /**
         * @description: 根据Http状态码处理响应数据
         * @param {AxiosResponse} res
         * @return {*}
         */
        function handleResponseStatus(resp) {
            if (!validateStatus || validateStatus(resp.status)) {
                resolve(resp);
            }
            else {
                reject(createError("Request failed with status code ".concat(resp.status), config, null, xhr, resp));
            }
        }
    });
}

/*
 * @Author: Lisc
 * @Date: 2022-04-07 17:41:20
 * @Description: 请求和响应数据转换函数 的处理逻辑
 */
function transform(data, headers, fns) {
    if (!fns) {
        return data;
    }
    // 将转换函数统一成数组形式
    if (!Array.isArray(fns)) {
        fns = [fns];
    }
    fns.forEach(function (fn) {
        data = fn(data, headers);
    });
    return data;
}

/**
 * @description:
 * @param {AxiosRequestConfig} config
 * @return {*}
 */
function dispatchRequest(config) {
    // cancelToken已经被使用过，再去携带相同的cancelToken发送请求没有意义
    throwIfCanceled(config);
    processRequestConfig(config);
    // 返回带有xhr请求结果的Promise对象
    return xhr(config).then(function (resp) {
        return transformResponseData(resp);
    }, function (e) {
        if (e && e.response) {
            e.response = transformResponseData(e.response);
        }
        return Promise.reject(e);
    });
}
function processRequestConfig(config) {
    config.url = transformUrl(config);
    config.data = transformRequestData(config);
    config.headers = flattenHeaders(config.headers, config.method);
}
/**
 * @description: 根据用户配置的baseURL，url，params和paramsSerializer得到完整的请求url
 * @param {AxiosRequestConfig} config
 * @return {*}
 */
function transformUrl(config) {
    var url = config.url, params = config.params, paramsSerializer = config.paramsSerializer, baseURL = config.baseURL;
    if (baseURL && !isAbsoluteURL(url)) {
        url = combineURL(baseURL, url);
    }
    return getUrlWithParams(url, params, paramsSerializer);
}
/**
 * @description: 根据用户设置的请求数据(请求体data)处理函数,对请求数据进行处理
 * 默认的请求数据处理函数为:如果请求数据为普通对象,将其转换成Json字符串,并设置其Content-Type为application/json
 * @param {AxiosRequestConfig} config
 * @return {*}
 */
function transformRequestData(config) {
    return transform(config.data, config.headers, config.transformRequest);
}
/**
 * @description: 根据用户设置的响应数据处理函数,对响应数据进行处理
 * 默认的响应数据处理函数为:如果响应数据为Json字符串,将其转换成Json对象
 * @param {AxiosRequestConfig} config
 * @return {*}
 */
function transformResponseData(resp) {
    resp.data = transform(resp.data, resp.headers, resp.config.transformResponse);
    return resp;
}
function throwIfCanceled(config) {
    if (config.cancelToken) {
        config.cancelToken.throwIfRequested();
    }
}

var InterceptorManager = /** @class */ (function () {
    function InterceptorManager() {
        this.interceptors = [];
    }
    /**
     * @description: 添加拦截器
     * @param {ResolvedFn} resolve
     * @param {RejectedFn} reject
     * @return {*}
     */
    InterceptorManager.prototype.use = function (resolve, reject) {
        this.interceptors.push({
            resolve: resolve,
            reject: reject
        });
        return this.interceptors.length - 1;
    };
    /**
     * @description: 删除拦截器
     * 注意这里不能直接修改原数组,因为id是数组下标,删除元素数组元素个数改变拦截器id会变化
     * @param {number} id
     * @return {*}
     */
    InterceptorManager.prototype.eject = function (id) {
        if (this.interceptors[id]) {
            this.interceptors[id] = null;
        }
    };
    /**
     * @description: 遍历拦截器,对每个拦截器执行回调函数
     * @param {function} fn
     * @return {*}
     */
    InterceptorManager.prototype.forEach = function (fn) {
        this.interceptors.forEach(function (interceptor) {
            if (interceptor) {
                fn(interceptor);
            }
        });
    };
    return InterceptorManager;
}());

/*
 * @Author: Lisc
 * @Date: 2022-04-06 18:37:46
 * @Description: 默认配置和用户配置合并
 */
var strategiesMap = Object.create(null);
/**
 * @description: 配置的默认合并策略,用户没有配置的字段,使用默认配置
 * @param {any} defaultVal
 * @param {any} userVal
 * @return {*}
 */
function defaultStrategy(defaultVal, userVal) {
    return typeof userVal === 'undefined' ? defaultVal : userVal;
}
/**
 * @description: 特殊字段配置合并策略,只使用用户的配置
 * @param {any} defaultVal
 * @param {any} userVal
 * @return {*}
 */
function fromUserStrategy(defaultVal, userVal) {
    if (typeof userVal !== 'undefined') {
        return userVal;
    }
}
/**
 * @description:
 * @param {any} defaultVal
 * @param {any} userVal
 * @return {*}
 */
function deepMergeStrategy(defaultVal, userVal) {
    if (isPlainObject(userVal)) {
        // 用户设置的值为对象,和默认配置对象合并
        return deepCopy(defaultVal, userVal);
    }
    // 用户没有设置合法值,使用默认配置（auth和headers不可能设置值类型的数据）
    else {
        return deepCopy(defaultVal);
    }
}
var fromUserStrategyKeys = ['url', 'params', 'data'];
fromUserStrategyKeys.forEach(function (key) {
    strategiesMap[key] = fromUserStrategy;
});
var deepMergeStrategyKeys = ['headers', 'auth'];
deepMergeStrategyKeys.forEach(function (key) {
    strategiesMap[key] = deepMergeStrategy;
});
function mergeConfig(defaultConfig, userConfig) {
    if (!userConfig) {
        userConfig = {};
    }
    var config = Object.create(null);
    for (var key in userConfig) {
        merge(key);
    }
    for (var key in defaultConfig) {
        if (!userConfig[key]) {
            merge(key);
        }
    }
    function merge(key) {
        // 不同的key对应不同的合并策略函数
        var strategy = strategiesMap[key] || defaultStrategy;
        // 使用合并策略函数进行合并
        config[key] = strategy(defaultConfig[key], userConfig[key]);
    }
    return config;
}

/*
 * @Author: Lisc
 * @Date: 2022-04-03 16:22:01
 * @Description: Axios类是Axios接口的底层实现,但是并不是严格意义的实现类
 * 因为request接口中只有config一个参数,而实现类有两个
 * 这样做是为了达到外界调用request方法时只能传入config对象的效果
 */
var Axios = /** @class */ (function () {
    function Axios(initConfig) {
        this.defaults = initConfig;
        this.interceptors = {
            request: new InterceptorManager(),
            response: new InterceptorManager(),
        };
    }
    /**
     * @description: 直接以函数形式(两种方式)调用axios 和 调用axios的别名(request,get,post等)时,底层的实现方法
     * @param {any} urlOrConfig
     * @param {AxiosRequestConfig} config
     * @return {*}
     */
    Axios.prototype.request = function (urlOrConfig, config) {
        // 支持直接传入配置对象和传入url和配置对象两种调用方式
        if (typeof urlOrConfig === 'string') {
            if (!config) {
                config = {};
            }
            config.url = urlOrConfig;
        }
        else {
            config = urlOrConfig;
        }
        config = mergeConfig(this.defaults, config);
        config.method = config.method.toLowerCase();
        //拦截器链式调用
        var chain = [
            // 初始化Promise链,最开始只有发送请求的resolve
            {
                resolve: dispatchRequest,
            },
        ];
        // 遍历拦截器并添加到Promise链
        this.interceptors.request.forEach(function (interceptor) {
            // 请求拦截器后添加的先执行
            chain.unshift(interceptor);
        });
        this.interceptors.response.forEach(function (interceptor) {
            // 响应拦截器先添加的先执行
            chain.push(interceptor);
        });
        //初始化一个resolve了config对象的promise对象
        //这里resolve了config对象,第一个then函数的resolve回调才能拿到config
        //并对config进行加工
        var promise = Promise.resolve(config);
        //链式调用
        while (chain.length) {
            var _a = chain.shift(), resolve = _a.resolve, reject = _a.reject;
            promise = promise.then(resolve, reject);
        }
        // 最终返回给用户的promise
        return promise;
    };
    Axios.prototype.get = function (url, config) {
        return this._requestWithoutData('get', url, config);
    };
    Axios.prototype.delete = function (url, config) {
        return this._requestWithoutData('delete', url, config);
    };
    Axios.prototype.head = function (url, config) {
        return this._requestWithoutData('head', url, config);
    };
    Axios.prototype.options = function (url, config) {
        return this._requestWithoutData('options', url, config);
    };
    Axios.prototype.post = function (url, data, config) {
        return this._requestWithData('post', url, data, config);
    };
    Axios.prototype.put = function (url, data, config) {
        return this._requestWithData('put', url, data, config);
    };
    Axios.prototype.patch = function (url, data, config) {
        return this._requestWithData('patch', url, data, config);
    };
    Axios.prototype.getUri = function (config) {
        config = mergeConfig(this.defaults, config);
        return transformUrl(config);
    };
    /**
     * @description: 支持get、delete、head和options方法
     * @param {Method} method
     * @param {string} url
     * @param {AxiosRequestConfig} config
     * @return {*}
     */
    Axios.prototype._requestWithoutData = function (method, url, config) {
        return this.request(Object.assign(config || {}, {
            method: method,
            url: url,
        }));
    };
    /**
     * @description: 支持post、put、patch方法
     * @param {*}
     * @return {*}
     */
    Axios.prototype._requestWithData = function (method, url, data, config) {
        return this.request(Object.assign(config || {}, {
            method: method,
            url: url,
            data: data,
        }));
    };
    return Axios;
}());

/*
 * @Author: Lisc
 * @Date: 2022-03-31 10:12:20
 * @Description: 处理请求体和响应体
 */
/**
 * @description: xhr不支持传递对象作为data数据，对象需要转成JSon字符串
 * @param {any} data
 * @return {*}
 */
function transformRequest(data) {
    if (isPlainObject(data)) {
        return JSON.stringify(data);
    }
    return data;
}
/**
 * @description: 如果返回response中的data是Json字符串，即使用户没有设置responseType为json
 * 也将其转换成Json对象方便操作
 * @param {any} data
 * @return {*}
 */
function transformResponse(data) {
    if (data && typeof data === 'string') {
        try {
            data = JSON.parse(data);
        }
        catch (_a) {
            // do nothing here
        }
    }
    return data;
}

/*
 * @Author: Lisc
 * @Date: 2022-04-06 14:16:28
 * @Description: Axios配置化的实现
 */
//请求的默认配置
var defaults = {
    method: 'get',
    timeout: 0,
    xsrfCookieName: 'XSRF-TOKEN',
    xsrfHeaderName: 'X-XSRF-TOKEN',
    headers: {
        common: {
            Accept: 'application/json,text/plain,*/*',
        },
    },
    transformRequest: [
        function (data, headers) {
            // 设置请求头
            setHeaders(headers, data);
            // 转换为Json字符串
            return transformRequest(data);
        },
    ],
    transformResponse: [
        function (data) {
            return transformResponse(data);
        },
    ],
    validateStatus: function (status) {
        return status >= 200 && status < 300; //|| status === 304
    },
};
var methodsWithData = ['post', 'put', 'patch'];
var methodsWithoutData = ['get', 'delete', 'head', 'options'];
methodsWithData.forEach(function (method) {
    defaults.headers[method] = {
        'Content-Type': 'application/x-www-form-urlencoded',
    };
});
methodsWithoutData.forEach(function (method) {
    defaults.headers[method] = {};
});

var Cancel = /** @class */ (function () {
    function Cancel(message) {
        this.message = message;
    }
    return Cancel;
}());
function isCancel(val) {
    return val instanceof Cancel;
}

/*
 * @Author: Lisc
 * @Date: 2022-04-09 14:46:05
 * @Description: CancelToken 类
 */
var CancelToken = /** @class */ (function () {
    // CancelToken 的构造函数参数支持传入一个 executor 函数，executor函数的参数也是一个函数（取消函数）
    function CancelToken(executor) {
        var _this = this;
        var resolveFn;
        this.promise = new Promise(function (resolve) {
            resolveFn = resolve;
        });
        // 外部调用cancel方法时的执行逻辑
        // 调用外部传入的executor函数，并为executor函数传入函数作为参数（函数即取消函数的逻辑）
        executor(function (message) {
            // 防止多次调用cancel方法
            if (_this.reason) {
                return;
            }
            _this.reason = new Cancel(message);
            resolveFn(_this.reason);
        });
    }
    CancelToken.prototype.throwIfRequested = function () {
        if (this.reason) {
            throw this.reason;
        }
    };
    /**
     * @description: CancelToken工厂方法
     * @param {*}
     * @return {*}
     */
    CancelToken.source = function () {
        var cancel;
        // 此处的c即为构造函数中执行executor时传入的函数参数
        // 从而完成了将类内的函数传递到类外调用的目的
        var token = new CancelToken(function (c) {
            cancel = c;
        });
        return {
            cancel: cancel,
            token: token
        };
    };
    return CancelToken;
}());

/**
 * @description: 创建Axios混合对象
 * @param {*}
 * @return {*}
 */
function createInstance(defaultConfig) {
    var context = new Axios(defaultConfig);
    var instance = Axios.prototype.request.bind(context);
    extend(instance, context);
    return instance;
}
var axios = createInstance(defaults);
axios.create = function (config) {
    return createInstance(mergeConfig(defaults, config));
};
axios.CancelToken = CancelToken;
axios.Cancel = Cancel;
axios.isCancel = isCancel;
axios.all = function (promises) {
    return Promise.all(promises);
};
axios.spread = function (callback) {
    return function wrap(arr) {
        return callback.apply(null, arr);
    };
};
axios.Axios = Axios;

/*
 * @Author: Lisc
 * @Date: 2022-03-30 15:48:07
 * @Description: 入口文件
 */

export { axios as default };
//# sourceMappingURL=ts-axios.es5.js.map
